/**
 * Copyright (C) 2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package com.funambol.syncclient.common;

import java.net.InetAddress;
import java.net.UnknownHostException;

import com.funambol.framework.tools.Base64;
import com.funambol.framework.tools.MD5;


/**
 * This class supplies a method for the creation of a device id based on
 * host address/name and username.
 *
 * @version  $Id: DeviceTools.java,v 1.2 2007/03/04 15:23:45 nichele Exp $
 */
public class DeviceTools {

    /**
     * Creates a device id as <code>prefix + B64(MD5(hostName:hostAddress:userName))</code>
     * @return device id as B64(MD5(hostName:hostAddress:userName)) or the default
     *         value if there are errors retrieving host address/name
     */
    public static String createDeviceID(String prefix, String defaultValue) {
        String userName    = System.getProperty("user.name");
        String hostName    = null;
        String hostAddress = null;
        InetAddress addr   = null;
        try {
            addr = InetAddress.getLocalHost();
        } catch (UnknownHostException ex) {
            return defaultValue;
        }
        hostName    = addr.getHostName();
        hostAddress = addr.getHostAddress();

        StringBuffer sb = new StringBuffer(hostName);
        sb.append(':').append(hostAddress).append(':').append(userName);
        String s = new String(Base64.encode(MD5.digest(sb.toString().getBytes())));

        return prefix + s;
    }

}
