/**
 * Copyright (C) 2003-2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package com.funambol.syncclient.common;

import java.io.File;
import java.io.FilenameFilter;

/**
 *
 *
 * @version $Id: ExtensionFilter.java,v 1.4 2007/01/11 19:21:40 nichele Exp $
 */
public class ExtensionFilter implements FilenameFilter {
    private String extension;

    public ExtensionFilter(String extension) {
        setExtension(extension);
    }

    public void setExtension(String extension) {
        if ((extension == null) || (extension.length() == 0)) {
            extension = ".*";
        }

        this.extension = (extension.startsWith("."))
                       ? extension
                       : '.' + extension
                       ;
        extension = extension.toLowerCase();
    }

    /**
     * Accepts:
     * <ul>
     *   <li>all directories
     *   <li>if extension is .* all files
     *   <li>if extension is not .* all files ending with extension
     * </ul>
     *
     * @param dir the file parent directory
     * @param name the file name
     *
     * @return true if the file must be included in the list, false otherwise
     */
    public boolean accept(File dir, String name) {
        if (extension.equals(".*")) {
            return true;
        }

        if (new File(dir, name).isDirectory()) {
            return true;
        }

        return name.toLowerCase().endsWith(extension);
    }
}
