/**
 * Copyright (C) 2003-2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package com.funambol.syncclient.common;

import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.net.URLConnection;
import java.net.MalformedURLException;

/**
 * This class supplies some methods of usefullness
 * for the management of HTTP connection.
 *
 * @version  $Id: HttpTools.java,v 1.4 2007/01/11 19:21:40 nichele Exp $
 *
 */

public class HttpTools {


    // -----------------------------------------------------------Public methods
    /**
     * Download file from the given location.
     *
     * @param fileUrl location from which download the file.
     * @return the content of the file downloaded.
     * @throws IOException if an I/O error occurs.
     */
    public static byte[] downloadPackage(String fileUrl)
    throws IOException {
        byte[] content = null;

        int size = -1;

        URL url = new URL(fileUrl);

        URLConnection urlConn = url.openConnection();
        url.openStream();
        size = urlConn.getContentLength();
        content = new byte[size];

        InputStream iStream =  url.openStream();
        int read = -1;
        int offset = 0;
        int toRead = size;
        while ( (read =  iStream.read(content, offset,toRead)) != -1) {
            toRead = toRead - read;
            offset = offset + read;
        }
        iStream.close();
        return content;

    }

    /**
     * Returns true if the given url is a valid http(s) url.
     *
     * @param url - NOT NULL
     *
     * @return true if <i>url</i>is a valid http(s) url, false otherwise
     */
    public static boolean isHTTPURL(String url) {
        if (!(url.startsWith("http://") || url.startsWith("https://"))) {
            return false;
        }
        
        try {
            new URL(url);
            return true;
        } catch (MalformedURLException e) {}
        
        return false;
    }


}