/**
 * Copyright (C) 2003-2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package com.funambol.syncclient.common;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.IOException;
import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;

/**
 * This class supplies some methods of usefullness for the
 * management of zip file.
 * <p>In particular it supplies methods for the check and for
 * the extraction of zip file
 *
 * @version $Id: ZipTools.java,v 1.4 2007/01/11 19:21:40 nichele Exp $
 *
 */
public class ZipTools {


    // ---------------------------------------------------------- Public methods
    /**
     * Extract the zip file rappresented by the given byte array.
     * @param workingDirectory the directory in which the file
     * should be extracted
     * @param zipFile the byte array that represents the zip
     * @throws Exception if error occurs or if the zip file is not valid
     */
    public static void extract(String workingDirectory, byte[] zipFile)
    throws Exception {

        ByteArrayInputStream byteStream = new ByteArrayInputStream(zipFile);
        ByteArrayOutputStream out = new ByteArrayOutputStream();

        ZipInputStream zipStream = new ZipInputStream(byteStream);
        ZipEntry zipEntry = null;

        String nameZipEntry = null;

        byte[] contentZipEntry = null;
        boolean isDirectory = false;

        int indexFileSeparator = -1;
        String directory = null;
        String fileName =  null;

        while ( (zipEntry = zipStream.getNextEntry()) != null ) {
            nameZipEntry = workingDirectory + File.separator + zipEntry.getName();

            isDirectory = zipEntry.isDirectory();

            if (isDirectory) {
                File file = new File(nameZipEntry);
                file.mkdirs();
            } else {

                // read zipEntry
                byte[] buf = new byte[1024];
                int c = 0;

                while ( (c = zipStream.read(buf)) != -1) {
                    out.write(buf, 0, c);
                }

                indexFileSeparator = nameZipEntry.lastIndexOf(File.separator);
                directory = nameZipEntry.substring(0, indexFileSeparator);
                fileName =  nameZipEntry.substring(indexFileSeparator+1,nameZipEntry.length());

                FileSystemTools.createFile(directory, fileName, out);

                out.reset();
                zipStream.closeEntry();
            }
        }
        zipStream.close();
        byteStream.close();
    }

    /**
     * Verify if the given byte array represents a valid zip file.
     * Reads all the content of the zip file but it does not save it
     * @param zipFile the byte array to verify
     * @throws IOException if the byte array does not represent a valid zip file
     */
    public static void verifyZip(byte[] zipFile) throws IOException {

        boolean isValidZipFile = true;

        ByteArrayInputStream bStream = new ByteArrayInputStream(zipFile);

        ZipInputStream zipStream = new ZipInputStream(bStream);

        ZipEntry zipEntry = null;

        String nameZipEntry = null;
        int dimZipEntry = 0;
        byte[] contentZipEntry = null;
        boolean isDirectory = false;

        while ( (zipEntry = zipStream.getNextEntry()) != null ) {

            isDirectory = zipEntry.isDirectory();

            if (!isDirectory) {
                // read zipEntry
                int c = -1;
                byte[] buf = new byte[1024];
                while ( (c = zipStream.read(buf)) != -1) {
                    dimZipEntry = dimZipEntry + c;
                }
                dimZipEntry = 0;
                zipStream.closeEntry();
            }
        }
        zipStream.close();
        bStream.close();
    }
}
