/**
 * Copyright (C) 2003-2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package com.funambol.syncclient.spap;

/**
 * This exception represents an error for the asset management.
 *
 * @version  $Id: AssetManagementException.java,v 1.4 2007/01/11 19:21:40 nichele Exp $
 *
 */
public class AssetManagementException extends Exception {


    private Asset asset = null;
    private String idAsset = null;
    private Throwable cause = null;

    // ------------------------------------------------------------ Constructors

    /**
     * Constructs an instance of <code>AssetManagementException</code> with the
     * specified detail message.
     *
     * @param message the detail message.
     */
    public AssetManagementException(String message) {
        super(message);
    }

    /**
     * Constructs an instance of <code>AssetManagementException</code> with the
     * specified detail message.
     *
     * and the cause.
     * @param message the detail message.
     * @param cause the cause of this error.
     */
    public AssetManagementException(String message, Throwable cause) {
        this(message);
        this.cause = cause;
    }

    /**
     * Constructs an instance of <code>AssetManagementException</code> with the
     * specified <code>Asset</code> and the specified message.
     *
     * @param asset the asset involved in this error.
     * @param message the detail message.
     */
    public AssetManagementException(Asset asset, String message) {
        super(message);
        this.asset = asset;
    }

    /**
     * Constructs an instance of <code>AssetManagementException</code> with
     * the specified <code>Asset</code> and the specified message and the cause.
     * @param asset the asset involved in this error.
     * @param message the detail message.
     * @param cause the cause of this error.
     */
    public AssetManagementException(Asset asset, String message, Throwable cause) {
        this(asset,message);
        this.cause = cause;
    }

    /**
     * Constructs an instance of <code>AssetManagementException</code> with
     * the specified idAsset and the specified message
     * @param idAsset the idAsset involved in this error.
     * @param message the detail message.
     */
    public AssetManagementException(String idAsset, String message) {
        super(message);
        this.idAsset = idAsset;
    }

    /**
     * Constructs an instance of <code>AssetManagementException</code> with the specified
     * idAsset and the specified message and the cause.
     * @param idAsset the idAsset involved in this error.
     * @param message the detail message.
     * @param cause the cause of this error.
     */
    public AssetManagementException(String idAsset, String message, Throwable cause) {
        this(idAsset,message);
        this.cause = cause;
    }

    //---------------------------------------------------
    //  Overrides java.lang.Object method
    //---------------------------------------------------

    public String toString() {
        StringBuffer message = new StringBuffer();
        if (asset!=null) {
            message.append(super.toString());
            message.append(" (id: ");
            message.append(asset.getId());
            message.append(", manufacturer: ");
            message.append(asset.getManufacturer());
            message.append(", name: ");
            message.append(asset.getName());
            message.append(")");
        } else if (idAsset != null) {
            message.append(super.toString());
            message.append(" (id: ");
            message.append(idAsset);
            message.append(")");
        } else {
            message.append(super.toString());
        }

        return message.toString();
    }

    /**
     * Prints the exception and its cause (if any) stack trace
     */
    public void printStackTrace() {
        super.printStackTrace();
        if (cause != null) {
            System.err.println("Caused by:");
            cause.printStackTrace();
        }
    }

}