/**
 * Copyright (C) 2003-2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package com.funambol.syncclient.spap.launcher;

import java.util.Hashtable;


import com.funambol.syncclient.spap.AssetInstallationException;
import com.funambol.syncclient.spdm.DMException;
import com.funambol.syncclient.spdm.DeviceManager;
import com.funambol.syncclient.spdm.SimpleDeviceManager;

/**
 * Manages <code>Laucher</code>
 * <p>Supplies the method for obtain the appropriate <code>Launcher</code>
 *  for a given program
 *
 * @version  $Id: LauncherFactory.java,v 1.4 2007/01/11 19:21:40 nichele Exp $
 *
 */

public class LauncherFactory {

    // Contain the mapping from program's extension to appropriate launcher.
    // It's inizilized through DM
    private static Hashtable fileExtensionMapping = null;

    static {
        // init LauncherFactory with DM
        DeviceManager dm  = SimpleDeviceManager.getDeviceManager();

        try {
            fileExtensionMapping =
                    dm.getManagementTree("").getNodeValues("spap/launcherFactoryConfig");
        }
        catch (DMException ex) {
            throw new IllegalStateException("Error in LauncherFactory config");
        }
    }


    // ---------------------------------------------------------- Public methods

    /**
     * Returns an instance of the appropriate <code>Launcher</code>
     * for the given <i>programName</i>
     * @param programName name of the program of which it wants
     * the <code>Launcher</code>
     * @return the appropriate <code>Launcher</code>
     * @throws Exception if the appropriate <code>Launcher</code> is not found
     *                   or if an error occurs during the inizialization
     */
    public static Launcher getLauncher(String programName) throws AssetInstallationException {
        Launcher launcher = null;

        int indexPoint = programName.lastIndexOf(".");
        String extension = programName.substring(indexPoint + 1, programName.length());

        String classLauncher = (String)fileExtensionMapping.get(extension);

        if (classLauncher != null) {

            try {
                launcher =  (Launcher)(Class.forName(classLauncher).newInstance());
            } catch (ClassNotFoundException ex) {
                throw new AssetInstallationException(
                        "Error in launcher inizialization for class '" +
                        classLauncher + "' (Class not found)"
                                );
            } catch (Exception ex) {
                throw new AssetInstallationException(
                        "Error in launcher inizialization for class '" +
                        classLauncher + "' (" + ex.getMessage() + ")");
            }

        } else {
            throw new AssetInstallationException("Launcher not defined for " +
                                extension + " program");
        }
        return launcher;
    }


}