/**
 * Copyright (C) 2003-2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package com.funambol.syncclient.spap.launcher;

import java.io.File;
import java.io.IOException;

import com.funambol.syncclient.common.logging.Logger;
import com.funambol.syncclient.spap.AssetInstallationException;
import com.funambol.syncclient.spap.installer.InstallationContext;

/**
 * This class represents a <code>Laucher</code> for the execution of
 * the win program.
 *
 * @version  $Id: Win32Launcher.java,v 1.4 2007/01/11 19:21:40 nichele Exp $
 *
 */

public class Win32Launcher implements Launcher {


    // -----------------------------------------------------------Private data

    private Logger logger = new Logger();

    // -----------------------------------------------------------Public methods

    /**
     * Executes the win program with given name.
     *
     * @param programName the program to execute.
     * @param install <code>true</code> if the program is the installation program,
     *  <code>false</code> if the program is the uninstallation program
     * @param ctx installation context information
     *
     * @throws AssetInstallationException if an error occurs during the execution
     */
    public int execute(String programName, boolean install, InstallationContext ctx)
            throws AssetInstallationException {

        String completeProgramPath = ctx.getWorkingDirectory() + File.separator + programName;
        if (logger.isLoggable(Logger.DEBUG)) {
            logger.debug("Win32Launcher - Execute " +
                         completeProgramPath        );
        }

        int exitState = -1;
        try {
            Process proc = Runtime.getRuntime().exec(
                    completeProgramPath, null, new File(ctx.getWorkingDirectory())
                    );

            proc.waitFor();

            exitState = proc.exitValue();

            if (logger.isLoggable(Logger.DEBUG)) {
                logger.debug("Process executed. Return state: " + exitState);
            }

        } catch (Exception e) {
            throw new AssetInstallationException("Error launching '" + completeProgramPath + "'", e);
        }

        return exitState;
    }


}
