/**
 * Copyright (C) 2003-2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package com.funambol.syncclient.spdm;



/**
 * This is a simple implementation of the Device interface. It is still quite
 * generic and will be replaced in the future with more interesting
 * implementations. For now it is a good trade-off between quick and dirty.
 *
 *
 * @version $Id: DeviceImpl.java,v 1.4 2007/01/11 19:21:40 nichele Exp $
 */
public class DeviceImpl
implements Device {

    // --------------------------------------------------------------- Constants

    /**
     * The management tree where the device information is stored
     */
    public static final String CONTEXT_DEVICE = "spdm";

    /**
     * The base directory configuration property
     */
    public static final String PROP_BASE_DIR = "baseDir";

    // ------------------------------------------------------------ Private data

    private DeviceManager dm;

    // ------------------------------------------------------------ Constructors

    public DeviceImpl(DeviceManager dm) {
        this.dm = dm;
    }

    // ---------------------------------------------------------- Public methods

    /**
     * Returns the base directory.
     *
     * @see Device
     *
     * @return the base directory as a string.
     *
     * @throws DMException if there is an error in determining the current
     *         working directory.
     */
    public String getBaseDirectory() throws DMException {
        return (String)dm.getManagementTree(CONTEXT_DEVICE).getValue(PROP_BASE_DIR);
    }

    /**
     * Sets the device base directory
     *
     * @param baseDir the new base directory.
     *
     * @throws DMException in case of error setting the base direcotry
     */
    public void setBaseDirectory(String baseDir) throws DMException {
        dm.getManagementTree(CONTEXT_DEVICE).setValue(PROP_BASE_DIR, baseDir);
    }

}
