/**
 * Copyright (C) 2003-2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package com.funambol.syncclient.spdm;

import com.funambol.syncclient.spdm.Device;

/**
 * This class represents an abstract device manager. The device manager has the
 * responsibility of handling the device management tree.
 *
 *
 *
 * @version  $Id: DeviceManager.java,v 1.4 2007/01/11 19:21:40 nichele Exp $
 */
public abstract class DeviceManager {

    public static DeviceManager getDeviceManager() {
        throw new RuntimeException ( "This operation cannot be executed on DeviceManager, " +
                                     "but should be done on one of its subclasses."         );

    };

    /**
     * Returns a <i>Device</i> object representing the device managed by this
     * DeviceManager.
     *
     * @return a <i>Device</i> object representing the device managed by this
     * device manager.
     */
    abstract public Device getDevice();

    /**
     * Returns the management tree starting from the given parent node
     *
     * @param parent the parent node
     *
     * @return the management tree starting from the given parent node
     */
    abstract public ManagementNode getManagementTree(String parent);

    /**
     * The same as </i>getManagementTree("")</i>
     *
     * @return </i>getManagementTree("")</i>
     */
    abstract public ManagementNode getManagementTree();

}
