/**
 * Copyright (C) 2003-2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package com.funambol.syncclient.spds.engine;

import java.io.*;

import java.security.Principal;

/**
 * This class represents a synchronization operation to perform on a
 * <i>SyncSource</i>.
 * <p>
 * A <i>SyncOperation</i> can represent the following actions
 * <ul>
 * <li>new
 * <li>delete
 * <li>udate
 * <li>conflict
 * <li>nop (do nothing)
 * </ul>
 *
 *
 * @version $Id: SyncOperation.java,v 1.4 2007/01/11 19:21:40 nichele Exp $
 */
public class SyncOperation {

    public static final char NEW      = 'N';
    public static final char DELETE   = 'D';
    public static final char UPDATE   = 'U';
    public static final char CONFLICT = 'O';
    public static final char NOP      = '-';

    private SyncItem syncItem = null;
    private char operation    = 0;

    /**
     * Rturns the item associated to the operation
     *
     * @return the <i>SyncItem</i> associated to the operation
     */
    public SyncItem getSyncItem() {
        return this.syncItem;
    }

    /**
     * Returns the operation this object represents.
     *
     * @return the operation this object represents.
     */
    public char getOperation() {
        return this.operation;
    }

    /**
     * Creates a new <i>SyncOperation</i> given the <i>SyncItem</i> and the
     * operation.
     *
     * @param syncItem the syncItem
     * @param operation the operation
     */
    public SyncOperation(SyncItem  syncItem,
                             char operation) {

            this.syncItem  = syncItem;
            this.operation = operation;
    }
}
