/**
 * Copyright (C) 2003-2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package com.funambol.syncclient.spds.engine;

import java.io.*;
import java.util.Hashtable;

import java.security.Principal;

import com.funambol.syncclient.common.StringTools;


/**
 * This class groups the configuration information of a SyncSource.
 *
 *
 *
 * @version $Id: SyncSourceDefinition.java,v 1.4 2007/01/11 19:21:40 nichele Exp $
 */
public class SyncSourceDefinition {

    // --------------------------------------------------------------- Constants

    public static final String CONFIG_NAME                 = "name"          ;
    public static final String CONFIG_CLASS                = "sourceClass"   ;
    public static final String CONFIG_URI                  = "sourceURI"     ;
    public static final String CONFIG_TYPE                 = "type"          ;
    public static final String CONFIG_LAST                 = "last"          ;
    public static final String CONFIG_DEFAULT_SYNC         = "sync"          ;
    public static final String CONFIG_SUPPORTED_SYNC_MODES = "syncModes"     ;

    // ------------------------------------------------------------ Private date

    private Hashtable properties = null;

    public String getName() {
        return (String)properties.get(CONFIG_NAME);
    }

    public String getSourceClass() {
        return (String)properties.get(CONFIG_CLASS);
    }

    public String getSourceUri() {
        return (String)properties.get(CONFIG_URI);
    }

    public String getType() {
        return (String)properties.get(CONFIG_TYPE);
    }

    public String getDefaultSync() {
        return (String)properties.get(CONFIG_DEFAULT_SYNC);
    }

    public String[] getSupportedSyncModes() {
        return StringTools.split((String)properties.get(CONFIG_SUPPORTED_SYNC_MODES));
    }

    public long getLastTimestamp() {
        String last = (String)properties.get(CONFIG_LAST);

        try {
            return (last == null) ? 0 : Long.parseLong(last);
        } catch (NumberFormatException e) {
            return 0;
        }
    }

    public Hashtable getProperties() {
        return properties;
    }

    public SyncSourceDefinition (Hashtable properties) {
        String sourceClass  = (String)properties.get(CONFIG_CLASS               );
        String uri          = (String)properties.get(CONFIG_URI                 );
        String className    = (String)properties.get(CONFIG_CLASS               );
        String name         = (String)properties.get(CONFIG_NAME                );
        String type         = (String)properties.get(CONFIG_TYPE                );
        String defaultSync  = (String)properties.get(CONFIG_DEFAULT_SYNC        );
        String syncModes    = (String)properties.get(CONFIG_SUPPORTED_SYNC_MODES);


        if (sourceClass == null) {
            throw new IllegalArgumentException("Missing 'sourceClass' in properties!");
        }

        if (uri == null) {
            throw new IllegalArgumentException("Missing 'uri' in properties!");
        }

        if (name == null) {
            throw new IllegalArgumentException("Missing 'name' in properties!");
        }

        if (type == null) {
            throw new IllegalArgumentException("Missing 'type' in properties!");
        }

        if (defaultSync == null) {
            properties.put(CONFIG_DEFAULT_SYNC, "two-way");
        }

        if (syncModes == null) {
            properties.put(CONFIG_SUPPORTED_SYNC_MODES, "two-way");
        }

        this.properties = properties;
    }
}
