/**
 * Copyright (C) 2003-2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package com.funambol.syncclient.spds.event;

import java.util.Date;

/**
 * Is used to notify the listeners
 * for the principal states.
 *
 *
 *
 * @version $Id: SyncEvent.java,v 1.4 2007/01/11 19:21:41 nichele Exp $
 */
public class SyncEvent {

    // -------------------------------------------------------- Constants

    /** The synchronization process start */
    public static final int SYNC_BEGIN           = 0  ;

    /** The synchronization process end */
    public static final int SYNC_END             = 1  ;

    /** A no blocking error occours */
    public static final int SYNC_ERROR           = 2  ;

    /** Send initialization message */
    public static final int SEND_INITIALIZATION  = 3  ;

    /** Send modification message   */
    public static final int SEND_MODIFICATION    = 4  ;

    /** Send finalization message   */
    public static final int SEND_FINALIZATION    = 5  ;

    // -------------------------------------------------------- Private data

    private Throwable  cause    = null ;
    private Date       date     = null ;
    private String     message  = null ;
    private int        type     = 0    ;

    // -------------------------------------------------------- Constructors

    /**
     * Creates a SyncEvent
     *
     * @param type the event type
     * @param date the date and time when the event occours
     */
    public SyncEvent (int type, long date) {

        this.type = type;
        this.date = new Date(date);

    }

    /**
     * Creates a SyncEvent
     *
     * @param type the event type
     * @param date the date and time when the event occours
     * @param message only if the event is a SYNC_ERROR, <p>null</p> otherwise
     * @param cause only if the event is a SYNC_ERROR, <p>null</p> otherwise
     */
    public SyncEvent (int        type     ,
                      long       date     ,
                      String     message  ,
                      Throwable  cause    ) {

            this.type    = type           ;
            this.date    = new Date(date) ;
            this.message = message        ;
            this.cause   = cause          ;

    }

    // -------------------------------------------------------- Public methods

    public Throwable getCause () {
        return cause;
    }

    public Date getDate       () {
        return date;
    }

    public String getMessage  () {
        return message;
    }

    public int getType        () {
        return type;
    }

    public void setCause    (Throwable cause  ) {
        this.cause   = cause   ;
    }

    public void setDate     (Date      date   ) {
        this.date    = date    ;
    }

    public void setMessage  (String    message) {
        this.message = message ;
    }

    public void setType     (int       type   ) {
        this.type    = type    ;
    }

    // -------------------------------------------------------- Private methods

}
