/**
 * Copyright (C) 2003-2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package com.funambol.syncclient.spds.event;

import com.funambol.syncclient.spds.engine.SyncItemKey;

/**
 * Is used to notify the listeners
 * when a item is added / updated / deleted
 * by a command received from the server
 * or when a item is added / updated / deleted
 * on by the client
 *
 *
 *
 * @version $Id: SyncItemEvent.java,v 1.4 2007/01/11 19:21:41 nichele Exp $
 */
public class SyncItemEvent {

    // -------------------------------------------------------- Constants

    /** A new item is added on the client */
    public static final int ITEM_ADDED_BY_SERVER    = 0  ;

    /** An item is deleted on the client */
    public static final int ITEM_DELETED_BY_SERVER  = 1  ;

    /** An item is updated on the client */
    public static final int ITEM_UPDATED_BY_SERVER  = 2  ;

    /** A new item is added by the client */
    public static final int ITEM_ADDED_BY_CLIENT    = 3  ;

    /** An item is deleted by the client */
    public static final int ITEM_DELETED_BY_CLIENT  = 4  ;

    /** An item is updated by the client */
    public static final int ITEM_UPDATED_BY_CLIENT  = 5  ;

    // -------------------------------------------------------- Private data

    private SyncItemKey  itemKey   = null ;
    private String       sourceUri = null ;
    private int          type      = 0    ;

    // -------------------------------------------------------- Constructors

    /**
     * Creates a SyncItemEvent
     *
     * @param type the event type
     * @param sourceUri the source the item belong to
     * @param itemKey the item key
     */
    public SyncItemEvent(int         type       ,
                         String      sourceUri  ,
                         SyncItemKey itemKey    ) {

        this.type      = type      ;
        this.sourceUri = sourceUri ;
        this.itemKey   = itemKey   ;

    }

    // -------------------------------------------------------- Public methods

    public SyncItemKey getItemKey () {
        return itemKey;
    }

    public String getSourceUri    () {
        return sourceUri;
    }

    public int getType            () {
        return type;
    }

    public void setItemKey       (SyncItemKey  itemKey     ) {
        this.itemKey    = itemKey  ;
    }

    public void setSourceUri     (String       sourceUri   ) {
        this.sourceUri = sourceUri ;
    }

    public void setType          (int          type        ) {
        this.type      = type      ;
    }

    // -------------------------------------------------------- Private methods

}
