/**
 * Copyright (C) 2003-2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package com.funambol.syncclient.spds.event;

import java.util.Date;

/**
 * Is used to notify the listeners
 * of the beginning and the end
 * of the synchronization
 * of a particolar syncsource.
 *
 *
 *
 * @version $Id: SyncSourceEvent.java,v 1.4 2007/01/11 19:21:41 nichele Exp $
 */
public class SyncSourceEvent {

    // -------------------------------------------------------- Constants

    /** SyncSource Synchronization begin */
    public static final int SYNC_BEGIN         = 0  ;

    /** SyncSource Synchronization end */
    public static final int SYNC_END           = 1  ;

    // -------------------------------------------------------- Private data

    private Date    date       = null ;
    private String  sourceUri  = null ;
    private int     syncMode   = 0    ;
    private int     type       = 0    ;

    // -------------------------------------------------------- Constructors

    /**
     * Creates a SyncSourceEvent
     *
     * @param type the event type
     * @param sourceUri the uri of the syncsource
     * @param syncMode the type of the performed sync
     * @param date the date and time of the start or the end of the sync
     */
    public SyncSourceEvent(int     type      ,
                           String  sourceUri ,
                           int     syncMode  ,
                           long    date      ) {

        this.type      = type           ;
        this.sourceUri = sourceUri      ;
        this.syncMode  = syncMode       ;
        this.date      = new Date(date) ;

    }

    // -------------------------------------------------------- Public methods

    public Date getDate        () {
        return date;
    }

    public String getSourceUri () {
        return sourceUri;
    }

    public int getSyncMode     () {
        return syncMode;
    }

    public int getType         () {
        return type;
    }

    public void setDate        (Date    date        ) {
        this.date = date;
    }

    public void setSourceUri   (String  sourceUri   ) {
        this.sourceUri = sourceUri;
    }

    public void setSyncMode    (int     syncMode    ) {
        this.syncMode = syncMode;
    }

    public void setType        (int     type        ) {
        this.type = type;
    }

    // -------------------------------------------------------- Private methods

}
