/**
 * Copyright (C) 2003-2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package com.funambol.syncclient.spds.event;

import com.funambol.syncclient.spds.engine.SyncItemKey;

/**
 * Is used to notify the listeners
 * received a status command.
 *
 *
 *
 * @version $Id: SyncStatusEvent.java,v 1.4 2007/01/11 19:21:41 nichele Exp $
 */
public class SyncStatusEvent {

    // -------------------------------------------------------- Constants

    /** Create status to send to the server */
    public static final int STATUS_TO_SEND      = 0  ;

    /** Received status from the server */
    public static final int STATUS_RECEIVED     = 1  ;


    // -------------------------------------------------------- Private data

    private int          type       = 0    ;
    private String       command    = null ;
    private int          statusCode = 0    ;
    private SyncItemKey  itemKey    = null ;
    private String       sourceUri  = null ;

    // -------------------------------------------------------- Constructors

    /**
     * Creates a SyncStatusEvent
     *
     * @param type the event type
     * @param command the command the status relates to
     * @param statusCode the status code
     * @param itemKey the key of the item this status relates to
     * @param sourceUri the uri of the syncsource
     */
    public SyncStatusEvent(int         type        ,
                           String      command     ,
                           int         statusCode  ,
                           SyncItemKey itemKey     ,
                           String      sourceUri   ) {

        this.type       = type       ;
        this.command    = command    ;
        this.statusCode = statusCode ;
        this.itemKey    = itemKey    ;
        this.sourceUri  = sourceUri  ;

    }

    // -------------------------------------------------------- Public methods

    public int getType            () {
        return type;
    }

    public String getCommand      () {
        return command;
    }

    public SyncItemKey getItemKey () {
        return itemKey;
    }

    public String getSourceUri    () {
        return sourceUri;
    }

    public int getStatusCode      () {
        return statusCode;
    }


    public void setType        (int          type       ) {
        this.type     = type          ;
    }

    public void setCommand     (String       command    ) {
        this.command     = command    ;
    }

    public void setItemKey     (SyncItemKey  itemKey    ) {
        this.itemKey     = itemKey    ;
    }

    public void setSourceUri   (String       sourceUri  ) {
        this.sourceUri   = sourceUri  ;
    }

    public void setStatusCode  (int          statusCode ) {
        this.statusCode  = statusCode ;
    }

    // -------------------------------------------------------- Private methods

}
