/**
 * Copyright (C) 2003-2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package com.funambol.syncclient.spds.event;

import java.util.Date;

/**
 * Is used to notify the listeners
 * of the sending of data to the server
 * and of the reception of data from the server.
 *
 *
 *
 * @version $Id: SyncTransportEvent.java,v 1.4 2007/01/11 19:21:41 nichele Exp $
 */
public class SyncTransportEvent {


    // -------------------------------------------------------- Constants

    /** Engine started to send data to the server */
    public static final int SEND_DATA_BEGIN                = 0 ;

    /** Engine has sent all data to the server */
    public static final int SEND_DATA_END                  = 1 ;

    /** Engine started to receive data from the server */
    public static final int RECEIVE_DATA_BEGIN             = 2 ;

    /** Engine is receiving data from the server */
    public static final int DATA_RECEIVED                  = 3 ;

    /** Engine has received all data from the server */
    public static final int RECEIVE_DATA_END               = 4 ;

    // -------------------------------------------------------- Private data

    private int data  = 0  ;
    private int type  = 0  ;

    // -------------------------------------------------------- Constructors

    /**
     * Creates a SyncTransportEvent
     *
     * @param type the event type
     * @param data the data length
     */
    public SyncTransportEvent(int type ,
                              int data ) {
        this.type = type ;
        this.data = data ;
    }

    // -------------------------------------------------------- Public methods

    public int getData  () {
        return data;
    }

    public int getType  () {
        return type;
    }

    public void setData     (int   data ) {
        this.data = data ;
    }

    public void setType     (int   type ) {
        this.type = type ;
    }

    // -------------------------------------------------------- Private methods

}
