/**
 * Copyright (C) 2003-2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package com.funambol.syncclient.spds.source;

import java.io.*;
import java.security.Principal;

import java.util.Date;
import java.util.HashMap;
import java.util.Vector;
import java.util.Enumeration;
import java.util.Properties;

import sync4j.framework.tools.Base64;

import com.funambol.syncclient.common.FileSystemTools;
import com.funambol.syncclient.common.SourceUtils;
import com.funambol.syncclient.spds.engine.*;
import com.funambol.syncclient.spds.SyncException;



/**
 * This class implements a dummy <i>SyncSource</i> that just displays the calls
 * to its methods
 *
 * @version $Id: SIFSyncSource.java,v 1.9 2007/01/11 19:21:41 nichele Exp $
 *
 */
public class SIFSyncSource extends FileSystemSyncSource implements SyncSource  {

    // --------------------------------------------------------------- Constants

    // -------------------------------------------------------------- Properties

    // ------------------------------------------------------------ Constructors

    /** Creates a new instance of AbstractSyncSource */
    public SIFSyncSource() {
        super();
    }

    // ---------------------------------------------------------- Public methods

    /*
    * @see SyncSource
    */
    public SyncItem setSyncItem(Principal principal, SyncItem syncItem)
    throws SyncException {

        char itemState = syncItem.getState();

        try {
            String fileName = null;
            if (itemState == SyncItemState.NEW) {
                fileName = getUniqueFileName();
            } else {
                fileName = syncItem.getKey().getKeyAsString();
            }

            byte[] fileContent =
                (byte[])syncItem.getPropertyValue(SyncItem.PROPERTY_BINARY_CONTENT);

            if (fileContent == null) {
                fileContent = new byte[0];
            }

            HashMap hashMap         = null;
            HashMap hashMapFromFile = null;

            File f = new File (getSourceDirectory(), fileName);

            if (isEncode() && fileContent.length > 0) {


                if (f.exists()) {
                    hashMapFromFile = SourceUtils.
                        xmlToHashMap(FileSystemTools.readFileString(f));

                    hashMap = SourceUtils.xmlToHashMap
                        (new String(Base64.decode(fileContent)));

                    hashMapFromFile.putAll(hashMap);

                    FileSystemTools.writeFile(
                        SourceUtils.hashMapToXml(hashMapFromFile), f);


                }
                else {

                    hashMapFromFile = new HashMap();

                    hashMap = SourceUtils.xmlToHashMap
                        (new String(Base64.decode(fileContent)));

                    hashMapFromFile.putAll(hashMap);

                    FileSystemTools.writeFile(
                        SourceUtils.hashMapToXml(hashMapFromFile), f);
                }

            } else {

                if (f.exists()) {

                    hashMapFromFile = SourceUtils.
                        xmlToHashMap(FileSystemTools.readFileString(f));

                    hashMap = SourceUtils.
                        xmlToHashMap(new String(fileContent));

                    hashMapFromFile.putAll(hashMap);

                    FileSystemTools.writeFile(
                        SourceUtils.hashMapToXml(hashMapFromFile), f);

                } else {

                    hashMapFromFile = new HashMap();

                    hashMap = SourceUtils.
                        xmlToHashMap(new String(fileContent));

                    hashMapFromFile.putAll(hashMap);

                    FileSystemTools.writeFile(
                        SourceUtils.hashMapToXml(hashMapFromFile), f);

                }

            }

            Date t = (Date) syncItem.getPropertyValue(SyncItem.PROPERTY_TIMESTAMP);

            f.setLastModified(roundTime(t.getTime()));

            setState(principal,
                     fileName,
                     SyncItemState.SYNCHRONIZED,
                     roundTime(t.getTime()));

            SyncItem newSyncItem =
                new SyncItemImpl(this, fileName, SyncItemState.NEW);

            newSyncItem.setProperties(syncItem.getProperties());

            return newSyncItem;
        } catch (IOException e) {
            throw new SyncException( "Error setting the item "
                                      + syncItem
                                      , e
                                      );
        }
        catch (Exception e) {
            throw new SyncException( "Error setting the hashmap in item "
                                     + syncItem
                                     , e
                                     );
        }
    }

    // --------------------------------------------------------- Private methods

    /**
     * Filters the SyncItems in the synchronization database (after a refresh)
     * based on the given principal, last sync timestamp and state (see
     * SyncItemState). If state is equals to UNKNOWN all items are returned.<br>
     * Note that the current implementation ignores the principal: data do not
     * depend on users.
     *
     * @param principal principal. null means any
     * @param since     last sync timestamp. null neans since ever
     * @param state     the state to use as filter
     *
     * @return an array of SyncItem objects whose state is equal to the given
     *         state.
     */
    protected SyncItem[] filterSyncItems(Principal principal,
                                         Date      since    ,
                                         char      state    ) {

        Vector syncItems = new Vector();

        long fileTimestamp,
             sinceTimestamp = (since == null) ? -1 : roundTime(since.getTime());

        Properties syncDatabase = updateSyncDatabase(principal, sinceTimestamp);

        SyncItem  syncItem      = null;
        String    fileName      = null;
        String    stateString   = null;
        char      fileState           ;
        for (Enumeration e = syncDatabase.keys(); e.hasMoreElements(); ) {
            fileName  = (String)e.nextElement();
            stateString = (String)syncDatabase.get(fileName);
            fileState = stateFromStateString(stateString);
            if ((state == SyncItemState.UNKNOWN) || (fileState == state)) {
                fileTimestamp = lastModifiedFromStateString(stateString);
                if (fileTimestamp > sinceTimestamp ) {
                    syncItem = new SyncItemImpl(this, fileName, fileState);
                    if (isEncode()){
                        syncItem.setProperty(
                            new SyncItemProperty(SyncItem.PROPERTY_BINARY_CONTENT         ,
                                                 Base64.encode(readFileContent(fileName)))
                        );
                        syncItem.setProperty(
                            new SyncItemProperty(SyncItem.PROPERTY_FORMAT,
                                                 SIFSyncSource.FORMAT_BASE64)
                        );
                    } else {
                        syncItem.setProperty(
                            new SyncItemProperty(SyncItem.PROPERTY_BINARY_CONTENT,
                                                 readFileContent(fileName)       )
                        );
                    }
                    syncItem.setProperty(
                        new SyncItemProperty(SyncItem.PROPERTY_TYPE,this.getType())
                    );
                    syncItems.addElement(syncItem);
                }
            }
        }  // next e

        SyncItem[] ret = new SyncItem[syncItems.size()];
        for (int i=0; i<ret.length; ++i) {
            ret[i] = (SyncItem)syncItems.elementAt(i);
        }

        return ret;
    }
}
