/**
 * Copyright (C) 2003-2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package com.funambol.syncclient.spds.source;

import java.security.Principal;
import java.util.Date;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;

import sync4j.framework.core.AlertCode;

import com.funambol.syncclient.sps.common.*;

import com.funambol.syncclient.spds.engine.*;
import com.funambol.syncclient.spds.SyncRecordFunction;
import com.funambol.syncclient.spds.SyncException;

/**
 * This class implements a <i>SPSSyncSource</i>.
 * It uses SPS API.
 *
  *
 *
 * @version $Id: SPSSyncSource.java,v 1.5 2007/01/11 19:21:41 nichele Exp $
 *
 */
public class SPSSyncSource implements SyncSource {

    // --------------------------------------------------------------- Private data
    private String  name                = null;
    private String  type                = null;
    private String  sourceClass         = null;
    private String  sourceURI           = null;
    private String  storeManager        = null;
    private String  storeManagerPackage = null;
    private String  storeVolume         = null;

    private int syncMode;

    /** enable / disable source syncronization */
    private boolean sync = false;

    /** set true if source changed */
    /** if it is false, no write dataStore to handle  */
    private boolean syncSourceChanged = false;

    private boolean sortDB = false;

    /** creator id of handle application */
    private String creatorId = null;

    /** data type of handle database */
    private String dataStoreType  = null;

    private DataStoreManager dataStoreManager = null;

    private DataStore dataStore = null;

    private ClassLoader classLoader = null;


    // ------------------------------------------------------------ Constructors

    /** Creates a new instance of SPSSyncSource */
    public SPSSyncSource() {

    }

    // ---------------------------------------------------------- Public methods

    public String getName() {
        return this.name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public String getType() {
        return this.type;
    }

    public void setType(String type) {
        this.type = type;
    }


    /** Getter for property creatorId
     * @return Value of creator id
     */
    public String getCreatorId() {
        return this.creatorId;
    }

    /** Setter for property creatorId.
     * @param creatorId New value of property creator id.
     */
    public void setCreatorId(String creatorId) {
        this.creatorId = creatorId;
    }

    /** Getter for property dataStoreType
     * @return Value of data store type
     */
    public String getdataStoreType() {
        return this.dataStoreType;
    }

    /** Setter for property dataStoreType.
     * @param dataStoreType New value of property data store sype.
     */
    public void setDataStoreType(String dataStoreType) {
        this.dataStoreType = dataStoreType;
    }

    /** Getter for property sourceClass.
     * @return Value of property sourceClass.
     */
    public String getSourceClass() {
        return this.sourceClass;
    }

    /** Setter for property sourceClass.
     * @param sourceClass New value of property sourceClass.
     */
    public void setSourceClass(String sourceClass) {
        this.sourceClass = sourceClass;
    }

    /** Getter for property uri.
     * @return Value of property uri.
     */
    public String getSourceURI() {
        return this.sourceURI;
    }

    /** Setter for property uri.
     * @param sourceURI New value of property sourceURI.
     */
    public void setSourceURI(String sourceURI) {

        this.sourceURI = sourceURI;
    }

    /** Getter for property sortDB.
     * @return Value of property sortDB.
     */
    public boolean getSortDB() {
        return this.sortDB;
    }

    /** Setter for property sortDB.
     * @param sortDB New value of property sortDB.
     */
    public void setSortDB(String sortDB) {

        if ((sortDB).equals("true")) {
            this.sortDB = true;
        } else {
            this.sortDB = false;
        }

    }

    /** Getter for property storeManager.
     * @return Value of property storeManager.
     */
    public String getStoreManager() {
        return this.storeManager;
    }

    /** Setter for property storeManager.
     * @param storeManager New value of property storeManager.
     */
    public void setStoreManager(String storeManager) {
        this.storeManager = storeManager;
    }

    /** Getter for property storeManagerPackage.
     * @return Value of property storeManagerPackage.
     */
    public String getStoreManagerPackage() {
        return this.storeManagerPackage;
    }

    /** Setter for property storeManagerPackage.
     * @param storeManagerPackage New value of property storeManagerPackage.
     */
    public void setStoreManagerPackage(String storeManagerPackage) {
        this.storeManagerPackage = storeManagerPackage;
    }


    /** Getter for property classLoader.
     * @return Value of property classLoader.
     */
    public ClassLoader getClassLoader() {
        return this.classLoader;
    }

    /** Setter for property classLoader.
     * @param classLoader New value of property classLoader.
     */
    public void setClassLoader(ClassLoader classLoader) {
        this.classLoader = classLoader;
    }

    /** Getter for property sync.
     * @return Value of property sync.
     */
    public boolean getSync() {
        return this.sync;
    }

    /** Setter for property sync.
     * @param sync New value of property sync.
     */
    public void setSync(String sync) {

        if ((sync).equals("true")) {
            this.sync = true;
        } else {
            this.sync = false;
        }

    }

    /** Getter for property storeVolume.
     * @return Value of property storeVolume.
     */
    public String getStoreVolume() {
        return this.storeVolume;
    }

    /** Setter for property sync.
     * @param storeVolume New value of property storeVolume.
     */
    public void setStoreVolume(String storeVolume) {
        this.storeVolume = storeVolume;
    }

    // -------------------------------------------------------------------------

    /**
     * Returns a string representation of this object.
     *
     * @return a string representation of this object.
     */
    public String toString() {
        StringBuffer sb = new StringBuffer(super.toString());

        sb.append(" - {name: ").append(getName()     );
        sb.append(" type: "   ).append(getType()     );
        sb.append(" uri: "    ).append(getSourceURI());
        sb.append("}"         );

        return sb.toString();
    }

    /**
     * Returns all items in the data store belonging to the given principal.
     *
     * @param principal the principal for which the data has to be considered
     *                  Null means all principals
     *
     * @return an array of all <i>SyncItem</i>s stored in this source. If there
     *         are no items an empty array is returned.
     *
     * @throws SyncException in case of error (for instance if the
     *         underlying data store runs into problems)
     */
    public SyncItem[] getAllSyncItems(Principal principal) throws SyncException {
        Vector records = null;

        try {
            records = dataStore.findAllRecords();
        } catch (DataAccessException e) {
            throw new SyncException(e.getMessage());
        }

        return recordsToSyncItems(dataStore, records);
    }

    /**
     * Returns all deleted items belonging to the given principal and deleted
     * after the given point in time.
     *
     * @param principal the principal for which the data has to be considered
     *                  Null means all principals
     * @param since consider the changes since this point in time. Null means
     *              all items regardless when they were changed.
     *
     * @return an array of keys containing the <i>SyncItem</i>'s key of the deleted
     *         items after the last synchronizazion. If there are no deleted
     *         items an empty array is returned.
     *
     * @throws SyncException in case of error (for instance if the
     *         underlying data store runs into problems)
     */
    public SyncItem[] getDeletedSyncItems(Principal principal,
                                          Date since    ) throws SyncException {
        Vector records = null;

        try {
            records = dataStore.findRecords('D', since);
        } catch (DataAccessException e) {
            throw new SyncException(e.getMessage());
        }

        return recordsToSyncItems(dataStore, records);

    }

    /**
     * Returns all new items belonging to the given principal and created
     * after the given point in time.
     *
     * @param principal the principal for which the data has to be considered
     *                  Null means all principals
     * @param since consider the changes since this point in . Null means
     *              all items regardless when they were changed.
     *
     * @return an array of items containing representing the newly created items.
     *         If there are no new items an empty array MUST BE returned.
     *
     * @throws SyncException in case of error (for instance if the
     *         underlying data store runs into problems)
     */
    public SyncItem[] getNewSyncItems(Principal principal,
                                      Date since    ) throws SyncException {
        Vector records = null;

        try {
            records = dataStore.findRecords('N', since);
        } catch (DataAccessException e) {
            throw new SyncException(e.getMessage());
        }

        return recordsToSyncItems(dataStore, records);
    }

    /**
     * Returns all updated items belonging to the given principal and modified
     * after the given point in time.
     *
     * @param principal the principal for which the data has to be considered
     *                  Null means all principals
     * @param since consider the changes since this point in time. Null means
     *              all items regardless when they were changed.
     *
     * @return an array of keys containing the <i>SyncItem</I>'s key of the updated
     *         items after the last synchronizazion. It MUST NOT return null for
     *         no keys, but instad an empty array.
     */
    public SyncItem[] getUpdatedSyncItems(Principal principal,
                                          Date since    ) throws SyncException {
        Vector records = null;

        try {
            records = dataStore.findRecords('U', since);
        } catch (DataAccessException e) {
            throw new SyncException(e.getMessage());
        }

        return recordsToSyncItems(dataStore, records);

    }

    /**
     * Removes a SyncItem given its key.
     *
     * @param principal the entity that wants to do the operation
     * @param syncItem  the item to remove
     *
     * @throws SyncException in case of error (for instance if the
     *         underlying data store runs into problems)
     */
    public void removeSyncItem(Principal principal, SyncItem syncItem) throws SyncException {

        syncSourceChanged = true;

        Record record = null;

        record = syncItemToRecord (dataStore, syncItem);

        try {
            dataStore.deleteRecord(record);
        } catch (DataAccessException e) {
            throw new SyncException(e.getMessage());
        }

    }

    /**
     * Replaces an existing <i>SyncItem</i> or adds a new <i>SyncItem</i> if it
     * does not exist. The item is also returned giving the opportunity to the
     * source to modify its content and return the updated item (i.e. updating
     * the id to the GUID).
     *
     * @param principal the entity that wants to do the operation
     * @param syncItem  the item to replace/add
     *
     * @return the inserted/updated item
     *
     * @throws SyncException in case of error (for instance if the
     *         underlying data store runs into problems)
     */
    public SyncItem setSyncItem(Principal principal, SyncItem syncItem) throws SyncException {

        syncSourceChanged = true;

        Record record = null;

        long keyBase = 0;

        record = syncItemToRecord (dataStore, syncItem);

        try {
            record = dataStore.storeRecord(record);
        } catch (DataAccessException e) {
            throw new SyncException(e.getMessage());
        }

        return recordToSyncItem(dataStore, record);

    }

    /**
     * Method implements start sync operations
     */
    public void beginSync(int syncMode) throws SyncException {

        this.syncMode = syncMode;

        Hashtable dataStoreProperties = null;

        dataStoreManager =
            DataStoreManagerFactory.getDataStoreManager(
                storeManager,
                storeManagerPackage,
                classLoader
            );

        if (dataStoreManager == null) {
           throw new SyncException( "Datastore manager "
                                  + storeManagerPackage
                                  + '.'
                                  + storeManager
                                  + DataStoreManagerFactory.DATASTORE_MANAGER_FILE_NAME_END
                                  + " not found"
                                  );
        }

        //
        // We get the datastore name from the sourceURI as the trailing part
        // after '/'
        //
        String storeName = sourceURI;
        int p = storeName.lastIndexOf('/');
        if ((p>=0) && (p<(storeName.length()-1))) {
            storeName = sourceURI.substring(p+1);
        }

        dataStore = dataStoreManager.getDataStore(storeName);

        if (dataStore == null) {
            throw new SyncException("Datastore " + storeName + " not found!");
        }

        dataStoreProperties = new Hashtable();

        if (this.creatorId != null) {
            dataStoreProperties.put("creatorId", this.creatorId);
        }

        if (this.dataStoreType != null) {
            dataStoreProperties.put("dataStoreType", this.dataStoreType);
        }

        if (this.storeVolume != null) {
            dataStoreProperties.put("dataStoreVolume", this.storeVolume);
        }

        dataStore.setDataStoreProperties(dataStoreProperties);

        try {
            dataStore.startDBOperations();
        } catch (DataAccessException e) {
            throw new SyncException(e.getMessage());
        }

        //
        // if the sync is a REFRESH_FROM_SERVER, the records currently in the
        // data store must be deleted
        //
        if (syncMode == AlertCode.REFRESH_FROM_SERVER) {
            try {
                emptyDataStore();
            } catch (DataAccessException e) {
                throw new SyncException(e.getMessage());
            }
        }
    }

    /**
     * Method implements end sync operations
     */
    public void commitSync() throws SyncException {

        Vector records = null;

        //
        // If the database must be sorted, we sort it and we rewrite import to
        // the datastore
        //
        if (syncSourceChanged && sortDB) {
            try {
                records = (Vector) dataStore.findAllRecords().clone();

                int l = records.size();

                for (int i=0; i < l; i++) {
                    dataStore.deleteRecord((Record) records.elementAt(i));
                }

                records = sortRecords(records);

                l = records.size();

                for (int i=0; i < l; i++) {
                    dataStore.storeRecord((Record) records.elementAt(i));
                }

                records = null;

            } catch (DataAccessException e) {
                throw new SyncException(e.getMessage());
            }
        }

        try {
            dataStore.commitDBOperations();
        } catch (DataAccessException e) {
            throw new SyncException(e.getMessage());
        }

        dataStore = null;

        return;

    }

    // ---------------------------------------------------------- Private method

    /**
     * Convert a Vector of Record into Array of SyncItem.
     *
     * @param dataStore
     * @param records
     *
     * @return an Array of SyncItem
     *
     */
    private SyncItem[] recordsToSyncItems(DataStore dataStore, Vector records) {
        SyncItem[] syncItems = null;

        syncItems = new SyncItem[records.size()];

        int l = records.size();

        for (int i=0; i < l; i++) {
            syncItems[i] = recordToSyncItem(dataStore, (Record) records.elementAt(i));
        }

        return syncItems;
    }

    /**
     * Convert a Record into a SyncItem.
     *
     * @param dataStore
     * @param record
     *
     * @return a SyncItem
     *
     */
    private SyncItem recordToSyncItem(DataStore dataStore, Record record) {
        StringBuffer xml = new StringBuffer();

        int positionKey       = 0;
        int positionTimestamp = 0;
        int positionState     = 0;

        int count = 0;

        positionKey        = record.getPositionKeyField();
        positionTimestamp  = SyncRecordFunction.getPositionTimestampField(record);
        positionState      = SyncRecordFunction.getPositionModificationTypeField(record);

        //
        // The content is stored in the <i>SyncItem.PROPERTY_BINARY_CONTENT</i>
        // content as XML (see the design document for details on the XML schema).
        //
        SyncItem syncItem = null;

        syncItem = new SyncItemImpl(
           this,
           record.getKey(),
           record.getString(positionState).charAt(0)
        );

        //
        // XML-ize the columns ignoring key, timestamp and state
        //
        xml.append("<record xmlns=\"xmlns://www.funambol.com/spds\">\n");


        int l = dataStore.getRecordMetadata().getFieldMetadata().length;

        for (int i=1; i <= l; i++) {

            if (i != positionTimestamp && i != positionState) {

                if (record.getString(i)!=null && record.getString(i).length() > 0) {
                    xml.append("<field" + i + ">"    );
                    xml.append(record.getString(i));
                    xml.append("</field" + i + ">\n" );
                }

            }

        }

        xml.append("</record>\n");

        syncItem.setProperty(
            new SyncItemProperty(SyncItem.PROPERTY_BINARY_CONTENT    ,
                             xml.toString().getBytes())
        );
        syncItem.setProperty(
            new SyncItemProperty(SyncItem.PROPERTY_TYPE,this.getType())
        );


        return syncItem;

    }

    /**
     * Convert a SyncItem to Record
     *
     * @param dataStore
     * @param syncItem
     *
     * @return a Record
     *
     */
    private Record syncItemToRecord (DataStore dataStore, SyncItem syncItem) throws SyncException {

        Record record = null;

        Hashtable   fields       = null;
        Enumeration fieldNumbers = null;

        String xml           = null;
        String timestamp     = null;
        String fieldNumber   = null;
        String fieldValue    = null;

        int positionKey           = 0;
        int positionTimestamp     = 0;
        int positionState         = 0;
        int numberOperationFields = 0;

        int j=0;

        record = new Record (dataStore, syncItem.getKey().getKeyAsString());

        positionKey           = record.getPositionKeyField();
        positionTimestamp     = SyncRecordFunction.getPositionTimestampField(record);
        positionState         = SyncRecordFunction.getPositionModificationTypeField(record);
        numberOperationFields = SyncRecordFunction.getNumberOperationFields(record);

        if(syncItem.getPropertyValue(SyncItem.PROPERTY_BINARY_CONTENT) != null) {

            xml = new String((byte[])syncItem.getPropertyValue(SyncItem.PROPERTY_BINARY_CONTENT));

            fields = xmlToValues(xml);

            fieldNumbers = fields.keys();

            while (fieldNumbers.hasMoreElements()) {

                fieldNumber = (String) fieldNumbers.nextElement();
                fieldValue  = (String) fields.get((Object)fieldNumber);

                try {
                    record.setField(Integer.parseInt(fieldNumber), fieldValue);
                } catch (NumberFormatException e) {
                    throw new SyncException("SPSSyncSource, error syncItemToRecord: " + e.getMessage());

                }
            }

            if (positionState !=-1 ) {
                record.setField(positionState, String.valueOf(syncItem.getState()));
            }


        }

        return record;
    }


    /**
     * Parse the XML content in the form
     * <p>
     * &lt;record&gt;
     *   &lt;field&gt;<i>value</i>&lt;/field&gt;
     *   ...
     * &lt;/record&gt;
     * <p>
     * and return an hashtable of fieldNumber, fieldValue
     *
     * @param xml the xml string to parse
     *
     * @return the hashtable of fieldNumber, fieldValue
     */
    private Hashtable xmlToValues(String xml) {

        Hashtable fields   = null;
        String fieldNumber = null;
        String fieldValue  = null;

        int offset = 0, end, positionEndFieldTag, len;

        fields = new Hashtable();

        len = xml.length();

        while (offset < len) {

            offset = xml.indexOf("<field", offset);

            if (offset < 0) {
                break;
            }

            offset += 6;

            positionEndFieldTag = xml.indexOf(">", offset);

            //case <fieldxxx/>  (field void)
            if (xml.substring(positionEndFieldTag - 1, positionEndFieldTag).equals("/")) {

                fieldNumber = xml.substring(offset, positionEndFieldTag - 1);
                fieldValue = "";
                offset = positionEndFieldTag + 1;

            //case <fieldxxx>value</fieldxxx>
            } else {

                fieldNumber = xml.substring(offset, positionEndFieldTag);
                end = xml.indexOf("</field" + fieldNumber + ">", positionEndFieldTag + 1);
                if (end < 0) {
                    break;
                }

                fieldValue = xml.substring(positionEndFieldTag + 1, end);

                offset = end + fieldNumber.length() + 8;

            }

            fields.put(fieldNumber, fieldValue);
        }

        return fields;
    }

    /**
     * Sorts the given records based on their keys.
     *
     * @param records to sort
     *
     * @return sort records
     */
    private Vector sortRecords(Vector records) {

        Record tmp1Record  = null;

        Record tmp2Record = null;

        for(int i = records.size() - 1 ; i > 0; i-- ) {

            int maxLoc = 0;

            for (int j = 1; j <= i; j++) {
                if (((Record) records.elementAt(j)).getKey().compareTo(((Record) records.elementAt(maxLoc)).getKey()) > 0) {
                   maxLoc = j;
                }
            }

            tmp1Record = (Record) records.elementAt(maxLoc);

            tmp2Record = (Record) records.elementAt(i);

            records.setElementAt(tmp2Record, maxLoc);

            records.setElementAt((Object) tmp1Record, i);

        }

        return records;
    }

    /**
     * Empties the data store
     *
     * @throws DataAccessException
     */
    private void emptyDataStore() throws DataAccessException {

        Vector records = (Vector)dataStore.findAllRecords().clone();

        int l = records.size();

        for (int i=0; i < l; i++) {
            dataStore.deleteRecord((Record) records.elementAt(i));
        }

        records = null;

    }
}
