/**
 * Copyright (C) 2003-2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */



package com.funambol.syncclient.spds.util;



/**

 * This class implements string parser by parserDelimiter.

 *

  *

 * @version $Id: StrParser.java,v 1.4 2007/01/11 19:21:41 nichele Exp $

 **/

public class StrParser {



    // ------------------------------------------------------------ Private data



    private int position;



    private String str         = null;

    private String parserDelim = null;





    //------------------------------------------------------------- Constructors



    /**

     * Constructs a string parser for the specified string by parserDelimiter.

     *

     * @param   str            string to be parsed

     * @param   parserDelim    delimiter

     **/

    public StrParser(String str, String parserDelim) {



        position = 0;



        this.str = str;

        this.parserDelim = parserDelim;



    }



    //------------------------------------------------------------- Public methods



    /**

     * Verify if available more elements about this StrParser.

     *

     * @return  <code>true</code> if available more elements.

     *          <code>false</code> if not available more elements.

     **/

    public boolean hasMoreElements() {



        /** low performance

        if ((position <= str.length()-1) && ( str.substring(position).indexOf(parserDelim) != -1

             || str.substring(position).length() > 0)) {

            return true;

        } else {

            return false;

        }

        */



        //high performance

        if (str.substring(position).indexOf(parserDelim) != -1

             || str.substring(position).length() > 0) {

            return true;

        } else {

            return false;

        }



    }





    /**

     * @return next element about this StrParser.

     */

    public String nextElement() {



        int positionOld;

        int positionTmp;



        positionOld = position;



        positionTmp = str.substring(position).indexOf(parserDelim);





        if (positionTmp != -1) {

            position = position + positionTmp + parserDelim.length();

            return str.substring(positionOld, position - parserDelim.length());

        } else {

            position = str.length();

            return str.substring(positionOld, position);

        }



    }





    /**

     * @return number of elements about this StrParser.

     */

    public int countElements() {



        int count = 0;

        int index = 0;



        String strTmp = null;



        strTmp = str;



        while (strTmp != null && strTmp.length() > 0) {

            count++;



            index = strTmp.indexOf(parserDelim);



            if (index == -1)

                break;



            strTmp = strTmp.substring(index + 1);

        }



        return count;

    }

}
