/**
 * Copyright (C) 2003-2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package com.funambol.syncclient.sps.common;

import java.util.Vector;
import java.util.Date;
import java.util.Hashtable;


/**
 * This class create DataStore
 * and provide method to call new record
 *
 *
 * $Id: DataStore.java,v 1.4 2007/01/11 19:21:41 nichele Exp $
 */
public abstract class DataStore {

    // ------------------------------------------------------------ Private data

    private String dataStoreName          = null;
    private RecordMetadata recordMetadata = null;
    private Hashtable dataStoreProperties = null;

    //------------------------------------------------------------- Constructors

    public DataStore (String dataStoreName, RecordMetadata recordMetadata) {
        this.dataStoreName = dataStoreName;
        this.recordMetadata = recordMetadata;
    }

    //----------------------------------------------------------- Public methods

    /** Getter for property dataStoreProperties
     * @return Value of Data Store Properties
     */
    public Hashtable getDataStoreProperties() {
        return this.dataStoreProperties;
    }

    /** Setter for property dataStoreProperties.
     * @param dataStoreProperties New value of data Store Properties.
     */
    public void setDataStoreProperties(Hashtable dataStoreProperties) {
        this.dataStoreProperties = dataStoreProperties;
    }


    public String getDataStoreName() {

        return this.dataStoreName;

    }

    public RecordMetadata getRecordMetadata() {

        return this.recordMetadata;

    }

    /**
     * create record
     *
     * @param key
     * @return record
     * @throws DataAccessException
     **/
    public abstract Record newRecord(String key)
    throws DataAccessException;

    /**
     * read record
     *
     * @param record
     * @throws DataAccessException
     **/
    public abstract Record readRecord(Record record)
    throws DataAccessException;

    /**
     * store record
     *
     * @param record
     * @throws DataAccessException
     **/
    public abstract Record storeRecord(Record record)
    throws DataAccessException;

    /**
     * delete record
     *
     * @param record
     * @throws DataAccessException
     **/
    public abstract void deleteRecord(Record record)
    throws DataAccessException;

    /**
     * return alla records of dataStore
     *
     * @return find records
     * @throws DataAccessException
     **/
    public abstract Vector findAllRecords()
    throws DataAccessException;

    /**
     * return a Vector of Record
     * found by state, last timestamp
     *
     * @param state state of record
     * @param since last timestamp
     * @return find records
     * @throws DataAccessException
     **/
    public abstract Vector findRecords(char state, Date since)
    throws DataAccessException;

    /**
     * return records of dataStore find by spsRecordFilter
     *
     * @param recordFilter filter
     * @return find records
     * @throws DataAccessException
     **/
    public abstract Vector findRecords(RecordFilter recordFilter)
    throws DataAccessException;

    /**
     * @return datastore next key
     * @throws DataAccessException
     **/
    public abstract long getNextKey()
    throws DataAccessException;

    /**
     * Method define start DB operations
     * @throws DataAccessException
     */
    public abstract void startDBOperations()
    throws DataAccessException;

    /**
     * Method define end DB operations
     * @throws DataAccessException
     */
    public abstract void commitDBOperations()
    throws DataAccessException;;

}
