/**
 * Copyright (C) 2003-2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package com.funambol.syncclient.sps.common;

import java.util.Hashtable;

import com.funambol.syncclient.common.logging.Logger;

/**
 * This class return dataStoreManager from Class.forName instance
 * about user class DataStoreImpl
 *
 * @version $Id: DataStoreManagerFactory.java,v 1.4 2007/01/11 19:21:41 nichele Exp $
 **/
public class DataStoreManagerFactory {

     //--------------------------------------------------------------- Constants

    public static final String DATASTORE_MANAGER_FILE_NAME_END  = "StoreManager";

    //------------------------------------------------------------- Private data

    private static Logger logger = new Logger();

    //----------------------------------------------------------- Public methods


    /**
     * The same as <code>getDataStoreManager(storeManager, storeManagerPackage, getClass().getClassLoader())</code>
     *
     * @return dataStoreManager
     */
    public static DataStoreManager getDataStoreManager(String storeManager, String storeManagerPackage) {
        return getDataStoreManager(
            storeManager, storeManagerPackage, DataStoreManager.class.getClassLoader()
        );
    }

    public static DataStoreManager getDataStoreManager(String      storeManager       ,
                                                       String      storeManagerPackage,
                                                       ClassLoader loader             ) {
        DataStoreManager dataStoreManager = null;

        try {
            String className = storeManagerPackage + '.' + storeManager + DATASTORE_MANAGER_FILE_NAME_END;
            if (logger.isLoggable(Logger.DEBUG)) {
                logger.debug("Creating a new instance of "  + className);
            }
            dataStoreManager = (DataStoreManager) (loader.loadClass(className)).newInstance();
        } catch (Exception e) {
            e.printStackTrace();
            if (logger.isLoggable(Logger.INFO)) {
                logger.info("Error in loading class: " + e.getMessage());
            }
        }

        return dataStoreManager;
    }


}
