/**
 * Copyright (C) 2003-2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package com.funambol.syncclient.sps.common;

import java.util.Vector;

import com.funambol.syncclient.sps.common.*;


/**
 * This interface implements methods
 * about search filter in database
 *
 *
 *
 * $Id: FindFilter.java,v 1.4 2007/01/11 19:21:41 nichele Exp $
 **/

public class FindFilter extends RecordFilter {


    // ------------------------------------------------------------ Private data

    private DataStore dataStore      =  null  ;
    private Vector    valueToFind    =  null  ;

    private boolean   include        =  false ;
    private boolean   caseSensitive  =  false ;

    //------------------------------------------------------------- Constructors

    /**
     * @param dataStore dataStore name
     * @param valueToFind vector contains fields value to find
     * @param include <code>true</code> valueToFind fields contains in record fields,
                      <code>false</code> valueToFind fields equals record fields

     * @param caseSensitive <code>true</code> case sensitive find,
     *                      <code>false</code> match case find
     *
     */

    public FindFilter (DataStore dataStore, Vector valueToFind, boolean include, boolean caseSensitive) {

        this.dataStore     = dataStore     ;
        this.valueToFind   = valueToFind   ;
        this.include       = include       ;
        this.caseSensitive = caseSensitive ;

    }

    //----------------------------------------------------------- Public methods

    /**
     * This method is about to check
     * is record about criteria in dataStore
     * @param record to check
     * @return <code>true</code>  if the record is accept about criteria
     *         <code>false</code> if the record is not accept about criteria
     */
    public boolean accept (Record record) {

        boolean test    = false ;
        String  valueR  = null  ;
        String  valueF  = null  ;

        for (int i=0, l = this.valueToFind.size(); i < l; i++) {

            valueR = record.getString(i);

            valueF = (String) this.valueToFind.elementAt(i);

            if(include && caseSensitive && valueF.length() > 0
                && (valueR.toUpperCase().indexOf(valueF.toUpperCase()) != -1)) {

                test = true;

                break;

            } else if(!include && caseSensitive && valueF.length() > 0
                && valueR.toUpperCase().equals(valueF.toUpperCase())) {

                test = true;

                break;

            } else if(!include && !caseSensitive && valueF.length() > 0
                && valueR.equals(valueF)) {

                test = true;

                break;

            } if(include && !caseSensitive && valueF.length() > 0
                && (valueR.indexOf(valueF) != -1)) {

                test = true;

                break;

            }

        }

        return test;

    }

}