/**
 * Copyright (C) 2003-2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package com.funambol.syncclient.sps.jdbc;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.sql.Statement;

import java.util.Date;
import java.util.Hashtable;
import java.util.Vector;

import com.funambol.syncclient.common.logging.Logger;

import com.funambol.syncclient.spdm.DMException;
import com.funambol.syncclient.spdm.ManagementNode;
import com.funambol.syncclient.spdm.SimpleDeviceManager;

import com.funambol.syncclient.sps.common.DataStore;
import com.funambol.syncclient.sps.common.Record;
import com.funambol.syncclient.sps.common.RecordMetadata;
import com.funambol.syncclient.sps.common.RecordFilter;

/**
 * This class create DataStore
 * and provide method to operate about record
 *
 *
 * $Id: DataStoreImpl.java,v 1.4 2007/01/11 19:21:41 nichele Exp $
 */
public class DataStoreImpl extends DataStore {

    //------------------------------------------------------------- Private data

    Logger logger = new Logger();

    //------------------------------------------------------------- Constructors

    public DataStoreImpl(String dataStoreName, RecordMetadata recordMetadata) {
        super(dataStoreName, recordMetadata);
    }

    //------------------------------------------------------------- Public methods

    /**
     * create record
     *
     * @param key
     * @return record
     **/
    public Record newRecord(String key) {

        Record record = null;
        record = new Record(this, key);
        return record;

    }

    /**
     * read record
     * @param record
     **/
    public Record readRecord(Record record) {

        String query     = null;
        Object params[]  = null;

        Vector dataRows  = null;
        Object[] dataRow = null;


        query = DataStoreSql.selectQuery(this, record);

        params = new Object[1];

        params[0] = (Object) record.getKey();

        dataRows = execPreparedSelect(query, params);

        for (int i=0; i<dataRows.size(); i++) {

            dataRow = (Object[]) dataRows.elementAt(i);

            for (int j=1; j <= dataRow.length; j++) {
                record.setField(j, (String) dataRow[j-1]);
            }
        }

        return record;

    }

    /**
     * store record
     * @param record
     **/
    public Record storeRecord(Record record) {

        String query     = null;
        Object params[]  = null;

        Vector dataRows  = null;
        Object[] dataRow = null;

        boolean findRecord = false;

        int recordSize = 0;

        query = DataStoreSql.selectQuery(this, record);

        params = new Object[1];

        params[0] = (Object) record.getKey();

        dataRows = execPreparedSelect(query, params);

        if (dataRows.size() > 0) {
            findRecord = true;
        }

        recordSize = this.getRecordMetadata().getFieldMetadata().length;

        params = new Object[recordSize];

        if (findRecord) {

            query = DataStoreSql.updateQuery(this, record);

            int i=0;

            for(i = 1; i < recordSize; i++) {

                //provvisorio, in attesa di correzione baco sul server
                if (record.getString(i+1) == null || record.getString(i+1).equals("null")) {
                    params[i-1] = null;
                } else {
                    params[i-1] = (Object) record.getString(i+1);
                }
            }

            //key is last param in update query
            params[i-1] = (Object) record.getString(1);


        } else {

            record.setKey(String.valueOf(getNextKey()));

            query= DataStoreSql.insertQuery(this, record);

            for(int i = 1; i <= recordSize; i++) {

                //provvisorio, in attesa di correzione baco sul server
                if (record.getString(i) == null || record.getString(i).equals("null")) {
                    params[i-1] = null;
                } else {
                    params[i-1] = (Object) record.getString(i);
                }
            }

        }

        execPreparedUpdate(query, params);

        return record;

    }

    /**
     * delete record
     * @param record
     **/
    public void deleteRecord(Record record) {

        String query     = null;
        Object params[]  = null;

        Vector dataRows  = null;
        Object[] dataRow = null;

        boolean findRecord = false;

        query = DataStoreSql.selectQuery(this, record);

        params = new Object[1];

        params[0] = (Object) record.getKey();

        dataRows = execPreparedSelect(query, params);

        if (dataRows.size() > 0) {
            findRecord = true;
        }

        if (findRecord) {
            query = DataStoreSql.deleteQuery(this, record);
            execPreparedUpdate(query, params);
        } else {
            //record non esistente
            return;
        }

        return;

    }

    /**
     * return alla records of dataStore
     *
     * @return find records
     **/
    public Vector findAllRecords() {

        Vector records = new Vector();

        Record record = null;

        Connection con         = null;
        Statement  stmt        = null;
        ResultSet  rs          = null;
        ResultSetMetaData rsmd = null;

        String query = null;

        try {

            con = getConnection();

            stmt = con.createStatement();

            query = DataStoreSql.allRecordsQuery(this);

            rs = stmt.executeQuery(query);

            rsmd = rs.getMetaData();

            while (rs.next()) {

                record = new Record(this, rs.getString(1));

                for (int i=2; i <= rsmd.getColumnCount(); i++) {
                    record.setField (i, rs.getString(i));
                }

              records.addElement(record);

            }

        } catch(Exception e) {
             if (logger.isLoggable(Logger.INFO)) {
                 logger.info("Error reading records: " + e.getMessage());
             }
        } finally {

            if(con != null) {
                try {con.close();} catch (Exception e){}
                con = null;
            }

            if(stmt != null) {
                try {stmt.close();} catch (Exception e){}
                stmt = null;
            }

            if(rs != null) {
                try{rs.close();} catch (Exception e){}
                rs = null;
            }

        }

        return records;

    }

    /**
     * return a Vector of Records
     * found by state, last timestamp
     *
     * @param state state of record
     * @param since last timestamp
     * @return find records
     **/
    public Vector findRecords(char state, Date since) {

        Vector records = new Vector();

        Record record = null;

        Connection con         = null;
        PreparedStatement ps   = null;
        Statement  stmt        = null;
        ResultSet  rs          = null;
        ResultSetMetaData rsmd = null;

        String query = null;

        String nameFieldState      = null;
        String nameFieldTimestamp  = null;

        String function = null;

        for (int i=0; i < this.getRecordMetadata().getFieldMetadata().length; i++) {

            function = (String) this.getRecordMetadata().getFieldMetadata()[i].getAttributes().get("function");

            if (state!=' ' && function.equals("mtype")) {
                nameFieldState = this.getRecordMetadata().getFieldMetadata()[i].getName();
            }

            if (since!=null && function.equals("timestamp")) {
                nameFieldTimestamp = this.getRecordMetadata().getFieldMetadata()[i].getName();
            }

        }

        try {

            con = getConnection();

            stmt = con.createStatement();

            query = DataStoreSql.RecordsByStateByDateQuery(this, nameFieldState, nameFieldTimestamp);

            ps = con.prepareStatement(query);

            if (nameFieldState != null && nameFieldTimestamp != null) {
                ps.setString(1, String.valueOf(state));
                ps.setDate(2, new java.sql.Date(since.getTime()));
            } else if (nameFieldState != null && nameFieldTimestamp == null) {
                ps.setString(1, String.valueOf(state));
            } else if (nameFieldState == null && nameFieldTimestamp != null) {
                ps.setDate(1, new java.sql.Date(since.getTime()));
            }

            rs = ps.executeQuery();

            rsmd = rs.getMetaData();

            while (rs.next()) {

                record = new Record(this, rs.getString(1));

                for (int i=2; i <= rsmd.getColumnCount(); i++) {
                    record.setField (i, rs.getString(i));
                }

              records.addElement(record);

            }

        } catch(Exception e) {
             if (logger.isLoggable(Logger.INFO) ||
                  logger.isLoggable(Logger.DEBUG)) {
                  logger.info("Error reading records: " + e.getMessage());
             }
        } finally {

            if(con != null) {
                try {con.close();} catch (Exception e){}
                con = null;
            }

            if(stmt != null) {
                try {stmt.close();} catch (Exception e){}
                stmt = null;
            }

            if(rs != null) {
                try{rs.close();} catch (Exception e){}
                rs = null;
            }

        }

        return records;

    }


    /**
     * return records of dataStore find by RecordFilter
     *
     * @param recordFilter filter
     * @return find records
     **/
    public Vector findRecords(RecordFilter recordFilter) {

        Vector allRecords  = null;
        Vector findRecords = new Vector();

        Record record = null;

        allRecords = this.findAllRecords();

        for(int i=0; i < allRecords.size(); i++) {

            record = this.readRecord((Record) allRecords.elementAt(i));

            if(recordFilter.accept(record)) {
                findRecords.addElement(allRecords.elementAt(i));
            }
        }

        return findRecords;

    }

    /**
     * @return datastore next key
     **/
    public long getNextKey() {

        Connection con         = null;
        Statement  stmt        = null;
        ResultSet  rs          = null;
        ResultSetMetaData rsmd = null;

        String query = null;

        String function = null;
        String nameFieldKey = null;

        int nRecords = 0;
        long key = 0;


        for (int i=0; i < this.getRecordMetadata().getFieldMetadata().length; i++) {

            function = (String) this.getRecordMetadata().getFieldMetadata()[i].getAttributes().get("function");

            if (function.equals("key")) {
                nameFieldKey = this.getRecordMetadata().getFieldMetadata()[i].getName();
            }

        }

        try {

            con = getConnection();

            stmt = con.createStatement();

            query = "select count(*) from " + this.getDataStoreName();

            rs = stmt.executeQuery(query);

            while (rs.next()) {

                try {
                    nRecords = Integer.parseInt(rs.getString(1));
                } catch (NumberFormatException e) {
                    if (logger.isLoggable(Logger.INFO)) {
                      logger.info("Error reading records: " + e.getMessage());
                    }
                }
            }

            if  (nRecords!=0) {

                query = DataStoreSql.getNextKeyQuery(this, nameFieldKey);

                rs = stmt.executeQuery(query);

                while (rs.next()) {
                    try {
                        key = Long.parseLong(rs.getString(1));
                    } catch (NumberFormatException e) {
                        if (logger.isLoggable(Logger.INFO)) {
                            logger.info("Error reading records: " + e.getMessage());
                        }
                    }
                }

            } else {
                //key about no record in table
                key = 1000;

            }


        } catch(Exception e) {
             if (logger.isLoggable(Logger.INFO)) {
                logger.info("Error reading records: " + e.getMessage());
             }
        } finally {

            if(con != null) {
                try {con.close();} catch (Exception e){}
                con = null;
            }

            if(stmt != null) {
                try {stmt.close();} catch (Exception e){}
                stmt = null;
            }

            if(rs != null) {
                try{rs.close();} catch (Exception e){}
                rs = null;
            }

        }

        return key;
    }

    //----------------------------------------------------------- Private methods

    /**
     * Execute update query by PreparedStatement
     *
     * @param query query formatted in PreparedStatement form
     * @param params params about query
     **/
    private void execPreparedUpdate(String query, Object[] params) {

        Connection con         = null;
        ResultSet  rs          = null;

        PreparedStatement preparedStatement;

        try {

            con = getConnection();

            preparedStatement = con.prepareStatement(query);

            int numParam = params.length;

            for (int i=0; i<numParam; i++) {
                preparedStatement.setObject(i+1, params[i]);
            }

            preparedStatement.executeUpdate();

        }  catch(Exception e) {
             if (logger.isLoggable(Logger.INFO)) {
                 logger.info("Error reading records: " + e.getMessage());
             }
        } finally {

            if(con != null) {
                try{con.close();} catch (Exception e){}
                con = null;
            }

        }

    }

    /**
     * Execute select query by PreparedStatement
     *
     * @param query query formatted in PreparedStatement form
     * @param params params about query
     * @return Vector about read records
     **/
    private Vector execPreparedSelect(String query, Object[] params) {

        Connection con         = null;
        ResultSet  rs          = null;

        Vector dataRows;
        Object[] dataRow = null;
        Object obj =null;

        dataRows = new Vector();


        try {

            con = getConnection();

            PreparedStatement preparedStatement = con.prepareStatement(query);

            int numParam = params.length;

            for (int i=0; i<numParam; i++) {
                preparedStatement.setObject(i+1, params[i]);
            }

            rs =  preparedStatement.executeQuery();

            int numCol = rs.getMetaData().getColumnCount();

            while (rs.next()) {

                dataRow = new Object[numCol];

                for (int i=0; i<numCol; i++) {
                  obj = rs.getObject(i+1);
                  dataRow[i] = obj;
                }

                dataRows.addElement(dataRow);

            }
        } catch(Exception e) {
             if (logger.isLoggable(Logger.INFO)) {
                logger.info("Error reading records: " + e.getMessage());
             }
        } finally {

            if(con != null) {
                try{con.close();} catch (Exception e){}
                con = null;
            }

        }

        return dataRows;

    }

    /**
     * @return database connection
     **/
    private Connection getConnection ()
    throws ClassNotFoundException, SQLException, DMException {

        String jdbcDriver         = null;
        String urlConnection      = null;
        String userConnection     = null;
        String passwordConnection = null;

        Connection con = null;

        String contextSources = "sps/jdbc";

        Hashtable jdbcParams = null;

        String appURI = null;

        ManagementNode rootNode    = null;
        ManagementNode sourcesNode = null;
        ManagementNode[] sources   = null;

        //@todo trovare un altro modo per settare la appURI
        //appURI = System.getProperty(SimpleDeviceManager.PROP_DM_APP_BASE);


        try {

            rootNode = SimpleDeviceManager.getDeviceManager().getManagementTree(appURI);

            sourcesNode = rootNode.getChildNode(contextSources);

            sources = sourcesNode.getChildren();

            jdbcParams = sources[0].getValues();

        } catch (DMException e) {
            if (logger.isLoggable(Logger.INFO)) {
                logger.info("Error reading jdbc config values: " + e.getMessage());
            }
        }

        jdbcDriver         = (String) jdbcParams.get("jdbcDriver");
        urlConnection      = (String) jdbcParams.get("url");
        userConnection     = (String) jdbcParams.get("user");
        passwordConnection = (String) jdbcParams.get("password");

        Class.forName(jdbcDriver);

        con = DriverManager.getConnection(urlConnection, userConnection, passwordConnection);

        return con;

    }

    /**
     * Method define start DB operathions
     */
    public void startDBOperations() {

    }

    /**
     * Method define end DB operathions
     */
    public void commitDBOperations() {

    }

}
