/**
 * Copyright (C) 2003-2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package com.funambol.syncclient.sps.palm;

import java.io.IOException;
import java.util.Date;
import java.util.Hashtable;
import java.util.Vector;

import com.funambol.syncclient.common.logging.Logger;

import com.funambol.syncclient.sps.common.DataAccessException;
import com.funambol.syncclient.sps.common.DataStore;
import com.funambol.syncclient.sps.common.Record;
import com.funambol.syncclient.sps.common.RecordFilter;
import com.funambol.syncclient.sps.common.RecordMetadata;
import com.funambol.syncclient.spds.SyncRecordFunction;

import palm.conduit.SyncException;
import palm.conduit.SyncManager;

/**
 * This class create implementation of Palm DataStore
 * and provide method to operate about record
 *
 *
 * $Id: PalmDataStore.java,v 1.4 2007/01/11 19:21:41 nichele Exp $
 */

public class PalmDataStore extends DataStore {

    private Vector  palmDB           = null  ;
    private boolean dataStoreChanged = false ;

    private Logger logger = new Logger();

    //----------------------------------------------------------- Costructors

    public PalmDataStore(String dataStoreName, RecordMetadata recordMetadata) {
        super(dataStoreName, recordMetadata);
    }

    //----------------------------------------------------------- Public methods

    /**
     * create record
     *
     * @param key
     * @return record
     **/
    public Record newRecord(String key) {

        Record record = null;
        record = new Record(this, key);
        return record;

    }

    /**
     * read record
     * @param record
     * @throws DataAccessException
     **/
    public Record readRecord(Record record)
    throws DataAccessException {

        try {

            int l = palmDB.size();

            for (int i=0; i < l; i++) {

                if(((Record)palmDB.elementAt(i)).getKey().equals(record.getKey())) {
                    return (Record)palmDB.elementAt(i);
                }
            }

        } catch (ArrayIndexOutOfBoundsException e) {
            throw new DataAccessException(e.getMessage());
        }

        return record;

    }

    /**
     * store record
     * @param record
     * @return stored Record
     * @throws DataAccessException
     **/
    public Record storeRecord(Record record)
    throws DataAccessException {

        boolean findRecord = false;

        dataStoreChanged = true;

        int i=0;

        try {

            int l = palmDB.size();

            for (i=0; i < l; i++) {

                if(((Record)palmDB.elementAt(i)).getKey().equals(record.getKey())) {
                    findRecord = true;
                    break;
                }

            }

            if(findRecord) {
                palmDB.setElementAt((Object) record, i);
            } else {

               //
               //enable this code to key generate by client
               //String key = String.valueOf(getNextKey());
               //record.setKey(key);
               //

               palmDB.addElement((Object) record);
            }

        } catch (ArrayIndexOutOfBoundsException e) {
            throw new DataAccessException(e.getMessage());
        }

        return record;

    }

    /**
     * delete record
     * @param record
     * @throws DataAccessException
     **/
    public void deleteRecord(Record record)
    throws DataAccessException {
        dataStoreChanged = true;

        try {

            int l = palmDB.size();

            for (int i=0; i < l; i++) {

                if(((Record)palmDB.elementAt(i)).getKey().equals(record.getKey())) {
                    palmDB.removeElementAt(i);
                    break;
                }
            }


        } catch (ArrayIndexOutOfBoundsException e) {
            throw new DataAccessException(e.getMessage());
        }

        return;

    }

    /**
     * return alla records of device
     *
     * @return find records
     * @throws DataAccessException
     **/
    public Vector initFindAllRecords() throws DataAccessException {

        Vector records = new Vector();

        String dbName = null;

        int countHHRecords = 0;

        int db = 0;

        boolean findRecord = false;

        Record record = null;

        PalmRecord palmRecord = new PalmRecord();

        dbName = this.getDataStoreName();

        boolean createDB = false;

        //
        // Check if exist Palm DB
        //
        try  {
            db = SyncManager.openDB(dbName, 0, SyncManager.OPEN_READ     |
                                               SyncManager.OPEN_WRITE    |
                                               SyncManager.OPEN_EXCLUSIVE  );
            SyncManager.closeDB(db);
        } catch (SyncException e) {
            createDB=true;
        }

        //
        // Create Palm DB (if not exist)
        //
        if (createDB) {

            try {
                db = SyncManager.
                        createDB(asciiStringToInt((String) this.
                                        getDataStoreProperties().get("creatorId")), 0x0,
                     (short)0, dbName, asciiStringToInt((String) this.
                                getDataStoreProperties().get("dataStoreType")));
                if (logger.isLoggable(Logger.DEBUG)) {
                    logger.debug("Creating DataStore: " + dbName);
                }

            } catch (SyncException e) {
                throw new DataAccessException("Error at init, creating DB " +
                                              dbName                        +
                                              ": "                          +
                                              e.getMessage()                );

            } catch (NumberFormatException e) {
                throw new DataAccessException("Error at init, number conversion error " +
                                              dbName                                    +
                                              ": "                                      +
                                              e.getMessage()                            );
            } finally {
                try {
                    SyncManager.closeDB(db);
                } catch (SyncException e) {
                    throw new DataAccessException("Error at init, closing DB " +
                                                  dbName                       +
                                                  ": "                         +
                                                  e.getMessage()               );
                }
            }

        }


        try {

            //
            // Open Palm DB
            //
            try {
                db = SyncManager.openDB(dbName, 0, SyncManager.OPEN_READ      |
                                                   SyncManager.OPEN_WRITE     |
                                                   SyncManager.OPEN_EXCLUSIVE   );
            } catch (palm.conduit.SyncException e) {
                throw new DataAccessException("Error at init, openening DB " +
                                              dbName                         +
                                              ": "                           +
                                              e.getMessage()                 );
            }

            try {
                countHHRecords = SyncManager.getDBRecordCount(db);
            } catch(IOException e) {
                throw new
                    DataAccessException("Error at init, recordCount not received: " +
                                        e.getMessage()                              );
            }

            for (int recordIndex=0; recordIndex < countHHRecords; recordIndex++) {

                palmRecord = new PalmRecord();

                palmRecord.setIndex(recordIndex);

                //
                //in PalmRecord key set in dedicate field
                //in Record key set at the first position of array
                //
                palmRecord.setRecordSize(this.getRecordMetadata().
                                            getFieldMetadata().length - 1);

                try {
                    SyncManager.readRecordByIndex(db, palmRecord);
                } catch(IOException e) {
                    throw new DataAccessException("Error, record at position: " +
                                                  recordIndex                   +
                                                  " not loaded: "               +
                                                  e.getMessage()                );
                }

                record = new Record(this, (String) palmRecord.getKey());

                int l = palmRecord.getSize();

                for (int i=0; i < l; i++) {
                    record.setField(i+2, (String) palmRecord.getStoreField(i));
                }

                records.addElement(record);
                record     = null ;
                palmRecord = null ;


            }

        } finally {
             try {
                SyncManager.closeDB(db);
             } catch (palm.conduit.SyncException e) {
                throw new DataAccessException("Error at init, closing DB " +
                                              dbName                       +
                                              ": "                         +
                                              e.getMessage()               );
             }
        }

        return records;

    }


    /**
     * return all records of device stored in memory
     *
     * @return find records
     **/
    public Vector findAllRecords() {

        return palmDB;
    }


    /**
     * return a Vector of Records
     * found by state
     *
     * @param state state of record
     * @param since last timestamp
     * @return find records
     * @throws DataAccessException
     **/
    public Vector findRecords(char state, Date since)
    throws DataAccessException {

        Vector findRecords = new Vector();

        try {

            int l = palmDB.size();

            for(int i=0; i < l; i++) {

                if (String.valueOf(state).equals(SyncRecordFunction.
                        getModificationType((Record) palmDB.elementAt(i))))  {
                    findRecords.addElement(palmDB.elementAt(i));
                }

            }

        } catch (ArrayIndexOutOfBoundsException e) {
            throw new DataAccessException(e.getMessage());
        }

        return findRecords;

    }


    /**
     * return records of dataStore find by RecordFilter
     *
     * @param recordFilter filter
     * @return find records
     * @throws DataAccessException
     **/
    public Vector findRecords(RecordFilter recordFilter)
    throws DataAccessException {

        Vector allRecords  = null;
        Vector findRecords = new Vector();


        Record record = null;

        allRecords = this.findAllRecords();

        try {

            int l = allRecords.size();

            for(int i=0; i < l; i++) {

                record = this.readRecord((Record) allRecords.elementAt(i));

                if(recordFilter.accept(record)) {
                    findRecords.addElement(allRecords.elementAt(i));
                }
            }

        } catch (ArrayIndexOutOfBoundsException e) {
            throw new DataAccessException(e.getMessage());
        }

        return findRecords;

    }


    /**
     * return next key on device
     * 1000 if no record exist
     *
     * @return key
     **/
    public long getNextKey() {

        String function     = null;
        String nameFieldKey = null;

        long recordKey = 0;
        long key = 0;

        //
        //for future implementation about key-field position
        //

        int l = this.getRecordMetadata().getFieldMetadata().length;

        for (int i=0; i < l; i++) {

            function = (String) this.getRecordMetadata().
                            getFieldMetadata()[i].getAttributes().get("function");

            if (function.equals("key")) {
                nameFieldKey = this.getRecordMetadata().getFieldMetadata()[i].getName();
            }

        }


        l = palmDB.size();

        if  (l != 0) {

            for (int i=0; i < l; i++) {

                recordKey = Long.parseLong(((Record) palmDB.elementAt(i)).getKey());

                if (recordKey > key) {
                    key = recordKey;
                }

            }

            key = key + 1;

        } else {

            //
            //key about no record in table
            //
            key = 1000;
        }

        return key;

    }


    /**
     * Method define start DB operathions
     * @throws DataAccessException
     */
    public void startDBOperations()
    throws DataAccessException {

        palmDB = initFindAllRecords();

    }


    /**
     * Method define end DB operathions
     * @throws DataAccessException
     */
    public void commitDBOperations()
    throws DataAccessException {

        int db = 0;

        String dbName = null;

        Vector palmRecords = null;

        PalmRecord palmRecord = null;

        Record record = null;

        //
        // dataStoreChanged tells us when anything changed at server side. If no
        // changes are detected from the server, we have to check if changes
        // where made locally. To do so, we check the record status flag
        //
        if (!dataStoreChanged) {
            char state;
            int l = palmDB.size();
            for (int i=0; i < l; ++i) {
                record = (Record) palmDB.elementAt(i);
                state = SyncRecordFunction.getModificationType(record).charAt(0);
                if (state != 'S') {
                    dataStoreChanged = true;
                    break;
                }
            }
            if (!dataStoreChanged) {
                // still not changed: nothing to do
                return;
            }
        }

        palmRecords = new Vector();

        int l = palmDB.size();

        for (int i=0; i < l; i++) {

            record = (Record) palmDB.elementAt(i);

            if (!"D".equals(SyncRecordFunction.getModificationType(record))) {
                SyncRecordFunction.setModificationType(record, "S");

                palmRecord = new PalmRecord();

                palmRecord.setKey(record.getKey());

                for (int j=2; j <= record.getLength(); j++) {
                    palmRecord.addStoreField(record.getString(j));
                }

                palmRecords.addElement(palmRecord);
            }

        }

        dbName = this.getDataStoreName();

        try {
            //
            // Open Palm DB
            //
            try {
                db = SyncManager.openDB(dbName, 0, SyncManager.OPEN_READ       |
                                                   SyncManager.OPEN_WRITE      |
                                                   SyncManager.OPEN_EXCLUSIVE   );
            } catch (palm.conduit.SyncException e) {
                throw new DataAccessException("Error at commit, opening DB " +
                                              dbName                         +
                                              ": "                           +
                                              e.getMessage()                 );
            }

            try {
                SyncManager.purgeAllRecs(db);
            } catch(SyncException e) {
                throw new
                    DataAccessException("Error at commit, all records not purged: " +
                                        e.getMessage()                              );
            }

            l = palmRecords.size();

            for(int i = 0; i < l; i++){
                palmRecord = (PalmRecord) palmRecords.elementAt(i);
                palmRecord.setId(0);
                palmRecord.setIndex(0);

                try {
                    SyncManager.writeRec(db, palmRecord);
                } catch(IOException e) {
                    throw new
                        DataAccessException("Error at commit, record not written: " +
                                            e.getMessage()                          );
                }
            }

            try {
                //
                //deletes all hh records marked as deleted
                //
                SyncManager.purgeDeletedRecs(db);
            } catch(SyncException e) {
                throw new
                    DataAccessException("Error at commit, deleted records not purged: " +
                    e.getMessage()                                                      );
            }

            try {
                //
                //reset all the sync flags on the hh
                //
                SyncManager.resetSyncFlags(db);
            } catch(SyncException e) {
                throw new
                    DataAccessException("Error at commit, flags not reset: " +
                    e.getMessage()                                           );
            }

        } finally {
            try {
                palmRecords = null;
                SyncManager.closeDB(db);
            } catch (palm.conduit.SyncException e) {
                throw new
                    DataAccessException("Error at commit, closing " +
                                        dbName                      +
                                        ": "                        +
                                        e.getMessage()              );
            }
        }

        return;

    }


    //----------------------------------------------------------- Private methods

    /**
     * Convert ascii string to int value
     *
     * @param  asciiValue value to convert
     * @return converted value
     * @throws NumberFormatException
     *
     */
    private int asciiStringToInt (String asciiValue)
        throws NumberFormatException {

        int intValue = 0;

        String hexValue = null;

        hexValue = "";

        int l = asciiValue.length();

        for (int i=0; i < l; i++) {
            hexValue += Integer.toHexString((new Character(asciiValue.charAt(i))).hashCode());
        }

        intValue = Integer.parseInt(hexValue, 16);

        return intValue;

    }

}
