/**
 * Copyright (C) 2003-2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

 /* Portions copyright  2001 Palm, Inc. or its subsidiaries. All rights reserved. */

package com.funambol.syncclient.sps.palm;

import java.io.*;
import java.util.Vector;

import palm.conduit.*;

/**
 * Extends GenericRecord to implement the Palm Record. This object can be
 * used to store and translate hand-held device records used by the 
 * Funambol Client application.
 *
 * $Id: PalmRecord.java,v 1.4 2007/01/11 19:21:41 nichele Exp $
 */
public class PalmRecord extends AbstractRecord {

    //---------------------------------------------------------------- Private data

    private int recordSize = 0;

    private String key   = null;
    private Vector store = new Vector();

    //---------------------------------------------------------------- Public methods

    public String getKey() {
        return this.key;
    }

    public String getStoreField(int i) {
        return (String) store.elementAt(i);
    }

    public int getSize() {
        return this.store.size();
    }

    public String getField(int position) {
        return (String) this.store.elementAt(position);
    }

    public int getRecordSize() {
        return this.recordSize;
    }

    public void setKey(String key) {
        this.key = key;
    }

    public void setField(int position, String value) {
        this.store.setElementAt((Object) value, position);
    }

    public void addStoreField(String field) {
        this.store.addElement(field);
    }

    public void setRecordSize(int recordSize) {
        this.recordSize = recordSize;
    }

    public void writeData(DataOutputStream out) throws IOException {
        String fieldValue = null;

        if (this.key != null) {
            out.write(key.getBytes());
            out.write(0);
        }

        int l = store.size();

        for (int i=0; i < l; i++) {
             fieldValue = (String) store.elementAt(i);

             if (fieldValue == null) {
                fieldValue = "";
             }

             out.write(fieldValue.getBytes());
             out.write(0);
        }
    }

    /** Converts a DataInputStream representing the body of the hand-held device's
     *  Funambol client record into PalmRecord object members.
     *  @param in The DataInputStream containing record body information from
     *  the hand-held device's Funambol client application record.
     */
    public void readData(DataInputStream in) throws IOException {
        Vector tmpStore = new Vector();

        this.key = readCString(in);

        int l = this.recordSize;

        for(int i=0; i < l; i++) {
            tmpStore.addElement(readCString(in));
        }

        this.store = tmpStore;
    }
}
