/**
 * Copyright (C) 2003-2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package com.funambol.syncclient.sps.pocketpc.util;

import java.io.*;
import java.util.StringTokenizer;

import com.funambol.syncclient.sps.pocketpc.RapiDB;

/**
 * This class imports a Windows CE database from a CSV file. Volume and database
 * names are passed on the command line. Optionally, the database is deleted and
 * recreated. The comma separated values are read from the standard input stream.
 *
 * @version $Id: CEDatabaseImport.java,v 1.4 2007/01/11 19:21:41 nichele Exp $
 */
public class CEDatabaseImport {


    /**
     * Syntax:
     *  com.funambol.syncclient.sps.common.util.CEDatabaseImport [-create] volume database key_field status_field
     * @param args the command line arguments
     */
    public static void main(String[] args) throws Exception {
        boolean create      = false;
        int     argc        = 0    ;
        int     keyField    = 1    ;
        int     statusField = 2    ;

        if (args.length < 4) {
            syntax();
            System.exit(-1);
        }

        if ((args.length == 5) && ("-create".equals(args[0]))) {
            create = true;
            argc = 1;
        }

        RapiDB db = new RapiDB(args[argc++], args[argc++]);

        keyField    = Integer.parseInt(args[argc++]);
        statusField = Integer.parseInt(args[argc++]);

        if (create) {
            if (db.openDB(keyField, statusField, keyField) == true) {
                db.deleteDB();
            }

            if (!db.createDB(keyField, statusField)) {
                System.out.println("Error creating the database!");
                System.exit(-1);
            }
        }

        if (!db.openDB(keyField, statusField, keyField)) {
            System.out.println("Error creating the database!");
            System.exit(-1);
        }

        BufferedReader reader = new BufferedReader( new InputStreamReader(System.in));

        int i=0;
        Object[] data;
        String key;
        String line;
        StringTokenizer stk;
        while((line = reader.readLine()) != null) {
            if (line.startsWith("#") || (line.trim().length() == 0)) {
                continue;
            }

            System.out.println("processing.... " + line);

            stk = new StringTokenizer(line, "|");

            //
            // First token is the id
            //
            key = stk.nextToken();

            data = new Object[stk.countTokens()];
            i = 0;
            while (stk.hasMoreTokens()) {
                data[i++] = stk.nextToken();
            }

            db.storeRecord(key, data);
        }

        db.closeDB();
        reader.close();
    }

    private static void syntax() {
        System.out.println("Wrong syntax.");
        System.out.println("Syntax:");
        System.out.println("com.funambol.syncclient.sps.common.util.CEDatabaseImport [-create] volume database");
    }

}
